(* ::Package:: *)

(*:Package Version: 7.01 *)

(*:Version History: 1.01 (2000.01.30 - 2006.01.28)
                    2.01 (2006.01.29 - 2006.02.25)
                    3.01 (2006.02.26 - 2006.03.04)
                    4.01 (2006.03.05 - 2006.12.20)
                    5.01 (2006.12.21 - 2007.07.07)
                    6.01 (2007.07.08 - 2009.10.16)
                    6.02 (2009.10.17 - 2010.01.09)
                    6.03 (2010.01.10 - 2015.01.06)
                    7.01 (2015.01.07 -     .  .  ) *)

(*:Copyright: Copyright 2000.01-2015.01, Souichiro Ikebe *)

(*:Name: SpecialFunctions`SpheroidalWave` *)

(*:Title: SpheroidalWave *)

(*:Keywords: Eigenvalue, FlammerWave, SpheroidalWave, StrattonWave, 
  FlammerWaveCos, SpheroidalWaveCos, StrattonWaveCos, Liouville *)

(*:Sources: 
  C. Flammar, Spheroidal wave functions, 
  Stanford University Press, California, (1957). 

  J. Meixner and F. W. Schefke, Mathieusche Funktionen und 
  Spheroidfunktionen, Springer Verlag, Berlin, Germany, (1954).

  M. Abramowitz and I. Stegun, Handbook of mathematical 
  functions with formulas, graphs, and mathematical 
  tables, Dover Publications, (1970).
  
  W. J. Thompson, Atlas for computing mathematical 
  functions, Wiley-Intersciencs Publication, (1997).
*)

(*:Summary:
  This package implements angular and radial 
  spheroidal wave functions for integer order. 
  It's solutions in used series of Legendre 
  and Bessel functions.
*)

(**************************************************************)


BeginPackage["SpecialFunctions`SpheroidalWave`"]



SpheroidalEigenvalueLambda::usage =
"SpheroidalEigenvalueLambda[nu,mu,gamma] gives the eigenvalues of 
 spheroidal wave functions with normalization in Meixner-Schafke scheme."

FlammerEigenvalueLambda::usage =
"FlammerEigenvalueLambda[m,n,c] gives the
 eigenvalues of spheroidal wave functions 
 with parameter c. Where, m and n is integer number.
 And normalization in Flammer scheme."

FlammerCoefficientD::usage =
"FlammerCoefficientD[m,n,c,r] gives the
 coefficients for Legendre and Bessel series 
 of spheroidal wave functions with parameter c. 
 Where, m and n is integer number.
 And normalization in Flammer scheme."

FlammerCoefficientDRho::usage =
"FlammerCoefficientDRho[m,n,c,r] gives the
 coefficients for Legendre series of spheroidal 
 wave functions of second kind with parameter c. 
 Where, m and n is integer number.
 And normalization in Flammer scheme."

JoiningFactorKappa1::usage =
"JoiningFactorKappa1[m,n,c] gives the joining constant
 of angular and radial spheroidal wave functions of 
 first kind for integer order with parameter c.
 And normalization in Flammer scheme."

JoiningFactorKappa2::usage =
"JoiningFactorKappa2[m,n,c] gives the joining constant
 of angular and radial spheroidal wave functions of 
 second kind for integer order with parameter c.
 And normalization in Flammer scheme."

FlammerWaveS1::usage =
"FlammerWaveS1[m,n,c,a,z] gives the angular
 spheroidal wave functions of first kind for 
 integer order with parameter c.
 And normalization in Flammer scheme.
 a=2 is Ferrers's type and a=3 is Hobson's type."

FlammerWaveS2::usage =
"FlammerWaveS2[m,n,c,a,z] gives the angular
 spheroidal wave functions of second kind for 
 integer order with parameter c.
 And normalization in Flammer scheme.
 a=2 is Ferrers's type and a=3 is Hobson's type."

FlammerWaveR1::usage =
"FlammerWaveR1[m,n,c,z] gives the radial
 spheroidal wave functions of first kind for 
 integer order with parameter c.
 And normalization in Flammer scheme."

FlammerWaveR2::usage =
"FlammerWaveR2[m,n,c,z] gives the radial
 spheroidal wave functions of second kind for 
 integer order with parameter c.
 And normalization in Flammer scheme."

FlammerWaveR3::usage =
"FlammerWaveR3[m,n,c,z] gives the radial
 spheroidal wave functions of third kind for 
 integer order with parameter c.
 And normalization in Flammer scheme."

FlammerWaveR4::usage =
"FlammerWaveR4[m,n,c,z] gives the radial
 spheroidal wave functions of fourth kind for 
 integer order with parameter c.
 And normalization in Flammer scheme."

SpheroidalWavePS::usage =
"SpheroidalWavePS[n,m,a,gamma,z] gives the
 angular spheroidal wave functions of first 
 kind for integer order with parameter gamma.
 And normalization in Meixner-Schafke scheme.
 a=2 is Ferrers's type and a=3 is Hobson's type."

SpheroidalWaveQS::usage =
"SpheroidalWaveQS[n,m,a,gamma,z] gives the
 angular spheroidal wave functions of second 
 kind for integer order with parameter gamma.
 And normalization in Meixner-Schafke scheme.
 a=2 is Ferrers's type and a=3 is Hobson's type."

SpheroidalWaveS1::usage =
"SpheroidalWaveS1[n,m,gamma,z] gives the radial
 spheroidal wave functions of first kind for 
 integer order with parameter gamma.
 And normalization in Meixner-Schafke scheme."

SpheroidalWaveS2::usage =
"SpheroidalWaveS2[n,m,gamma,z] gives the radial
 spheroidal wave functions of second kind for 
 integer order with parameter gamma.
 And normalization in Meixner-Schafke scheme."

SpheroidalWaveS3::usage =
"SpheroidalWaveS3[n,m,gamma,z] gives the radial
 spheroidal wave functions of third kind for 
 integer order with parameter gamma.
 And normalization in Meixner-Schafke scheme."

SpheroidalWaveS4::usage =
"SpheroidalWaveS4[n,m,gamma,z] gives the radial
 spheroidal wave functions of fourth kind for 
 integer order with parameter gamma.
 And normalization in Meixner-Schafke scheme."

StrattonWaveS1::usage =
"StrattonWaveS1[m,n,c,a,z] gives the
 angular spheroidal wave functions of first 
 kind for integer order with parameter c.
 And normalization in Stratton-Morse-Chu-
 Little-Corbato scheme.
 a=2 is Ferrers's type and a=3 is Hobson's type."

StrattonWaveS2::usage =
"StrattonWaveS2[m,n,c,a,z] gives the
 angular spheroidal wave functions of second 
 kind for integer order with parameter c.
 And normalization in Stratton-Morse-Chu-
 Little-Corbato scheme.
 a=2 is Ferrers's type and a=3 is Hobson's type."

FlammerWaveS1Cos::usage =
"FlammerWaveS1Cos[m,n,c,z] gives the angular
 spheroidal wave cosine-functions of first kind for 
 integer order with parameter c.
 And normalization in Flammer scheme."

FlammerWaveS2Cos::usage =
"FlammerWaveS2Cos[m,n,c,z] gives the angular
 spheroidal wave cosine-functions of second kind for 
 integer order with parameter c.
 And normalization in Flammer scheme."

FlammerWaveR1Cos::usage =
"FlammerWaveR1Cos[m,n,c,z] gives the radial
 spheroidal wave cosine functions of first kind for 
 integer order with parameter c.
 And normalization in Flammer scheme."

FlammerWaveR2Cos::usage =
"FlammerWaveR2Cos[m,n,c,z] gives the radial
 spheroidal wave cosine functions of second kind for 
 integer order with parameter c.
 And normalization in Flammer scheme."

FlammerWaveR3Cos::usage =
"FlammerWaveR3Cos[m,n,c,z] gives the radial
 spheroidal wave cosine functions of third kind for 
 integer order with parameter c.
 And normalization in Flammer scheme."

FlammerWaveR4Cos::usage =
"FlammerWaveR4Cos[m,n,c,z] gives the radial
 spheroidal wave cosine functions of fourth kind for 
 integer order with parameter c.
 And normalization in Flammer scheme."

SpheroidalWavePSCos::usage =
"SpheroidalWavePSCos[n,m,gamma,z] gives the
 angular spheroidal wave cosine functions of first 
 kind for integer order with parameter gamma.
 And normalization in Meixner-Schafke scheme."

SpheroidalWaveQSCos::usage =
"SpheroidalWaveQSCos[n,m,gamma,z] gives the
 angular spheroidal wave cosine functions of second 
 kind for integer order with parameter gamma.
 And normalization in Meixner-Schafke scheme."

SpheroidalWaveS1Cos::usage =
"SpheroidalWaveS1Cos[n,m,gamma,z] gives the radial
 spheroidal wave cosine functions of first kind for 
 integer order with parameter gamma.
 And normalization in Meixner-Schafke scheme."

SpheroidalWaveS2Cos::usage =
"SpheroidalWaveS2Cos[n,m,gamma,z] gives the radial
 spheroidal wave cosine functions of second kind for 
 integer order with parameter gamma.
 And normalization in Meixner-Schafke scheme."

SpheroidalWaveS3Cos::usage =
"SpheroidalWaveS3Cos[n,m,gamma,z] gives the radial
 spheroidal wave cosine functions of third kind for 
 integer order with parameter gamma.
 And normalization in Meixner-Schafke scheme."

SpheroidalWaveS4Cos::usage =
"SpheroidalWaveS4Cos[n,m,gamma,z] gives the radial
 spheroidal wave cosine functions of fourth kind for 
 integer order with parameter gamma.
 And normalization in Meixner-Schafke scheme."

StrattonWaveS1Cos::usage =
"StrattonWaveS1Cos[m,n,c,z] gives the
 angular spheroidal wave cosine functions of first 
 kind for integer order with parameter c.
 And normalization in Stratton-Morse-Chu-Little-
 Corbato scheme."

StrattonWaveS2Cos::usage =
"StrattonWaveS2Cos[m,n,c,z] gives the
 angular spheroidal wave cosine functions of second 
 kind for integer order with parameter c.
 And normalization in Stratton-Morse-Chu-Little-
 Corbato scheme."

LiouvilleP::usage =
"LiouvilleP[m,n,c,z] gives the Liouville's type of
 angular spheroidal wave cosine functions of first 
 kind for integer order with parameter c.
 And normalization in Flammer scheme."

LiouvilleQ::usage =
"LiouvilleQ[m,n,c,z] gives the Liouville's type of
 angular spheroidal wave cosine functions of second 
 kind for integer order with parameter c.
 And normalization in Flammer scheme."

LiouvilleJ::usage =
"LiouvilleJ[m,n,c,z] gives the Liouville's type of
 radial spheroidal wave hyperbolic cosine functions 
 of first kind for integer order with parameter c.
 And normalization in Flammer scheme."

LiouvilleY::usage =
"LiouvilleY[m,n,c,z] gives the Liouville's type of
 radial spheroidal wave hyperbolic cosine functions 
 of second kind for integer order with parameter c.
 And normalization in Flammer scheme."

LiouvilleH1::usage =
"LiouvilleH1[m,n,c,z] gives the Liouville's type of
 radial spheroidal wave hyperbolic cosine functions 
 of third kind for integer order with parameter c.
 And normalization in Flammer scheme."

LiouvilleH2::usage =
"LiouvilleH2[m,n,c,z] gives the Liouville's type of
 radial spheroidal wave hyperbolic cosine functions 
 of fourth kind for integer order with parameter c.
 And normalization in Flammer scheme."



(* Option *)

BranchCut::usage =
"BranchCut->1 or 2 is an option of FlammerWaveR2Cos,
 FlammerWaveR3Cos, FlammerWaveR4Cos, SpheroidalWaveS2Cos, 
 SpheroidalWaveS3Cos, SpheroidalWaveS4Cos, LiouvilleJ, LiouvilleY, 
 LiouvilleH1, LiouvilleH2.
 That determined the type of branch cut of radial function. 
 1 for lines on the one type, 2 for parallel lines type."

Options[FlammerWaveR2Cos]={BranchCut->1}

Options[FlammerWaveR3Cos]={BranchCut->1}

Options[FlammerWaveR4Cos]={BranchCut->1}

Options[SpheroidalWaveS2Cos]={BranchCut->1}

Options[SpheroidalWaveS3Cos]={BranchCut->1}

Options[SpheroidalWaveS4Cos]={BranchCut->1}

Options[LiouvilleJ]={BranchCut->2}

Options[LiouvilleY]={BranchCut->2}

Options[LiouvilleH1]={BranchCut->2}

Options[LiouvilleH2]={BranchCut->2}



Begin["SpecialFunctions`SpheroidalWave`Private`"]



Attributes[SpheroidalEigenvalueLambda]={Listable}

Attributes[setPrecSph]={Listable}

Attributes[FlammerEigenvalueLambda]={Listable}

Attributes[FlammerCoefficientD]={Listable}

Attributes[FlammerCoefficientDRho]={Listable}

Attributes[JoiningFactorKappa1]={Listable}

Attributes[JoiningFactorKappa2]={Listable}

Attributes[FlammerWaveS1]={Listable}

Attributes[FlammerWaveS2]={Listable}

Attributes[FlammerWaveR1]={Listable}

Attributes[FlammerWaveR2]={Listable}

Attributes[FlammerWaveR3]={Listable}

Attributes[FlammerWaveR4]={Listable}

Attributes[SpheroidalWavePS]={Listable}

Attributes[SpheroidalWaveQS]={Listable}

Attributes[SpheroidalWaveS1]={Listable}

Attributes[SpheroidalWaveS2]={Listable}

Attributes[SpheroidalWaveS3]={Listable}

Attributes[SpheroidalWaveS4]={Listable}

Attributes[StrattonWaveS1]={Listable}

Attributes[StrattonWaveS2]={Listable}

Attributes[FlammerWaveS1Cos]={Listable}

Attributes[FlammerWaveS2Cos]={Listable}

Attributes[FlammerWaveR1Cos]={Listable}

Attributes[FlammerWaveR2Cos]={Listable}

Attributes[FlammerWaveR3Cos]={Listable}

Attributes[FlammerWaveR4Cos]={Listable}

Attributes[SpheroidalWavePSCos]={Listable}

Attributes[SpheroidalWaveQSCos]={Listable}

Attributes[SpheroidalWaveS1Cos]={Listable}

Attributes[SpheroidalWaveS2Cos]={Listable}

Attributes[SpheroidalWaveS3Cos]={Listable}

Attributes[SpheroidalWaveS4Cos]={Listable}

Attributes[StrattonWaveS1Cos]={Listable}

Attributes[StrattonWaveS2Cos]={Listable}

Attributes[LiouvilleP]={Listable}

Attributes[LiouvilleQ]={Listable}

Attributes[LiouvilleJ]={Listable}

Attributes[LiouvilleY]={Listable}

Attributes[LiouvilleH1]={Listable}

Attributes[LiouvilleH2]={Listable}



(*****************************************************************************)
(*                              test functions                               *)
(*****************************************************************************)

intNQ[n_]:=
  If[N[Re[n]]===N[Round[Re[n]]] && N[Im[n]]===0.,True,False];



(*****************************************************************************)
(*                    SpheroidalEigenvalueLambda[nu,mu,g]                    *)
(*****************************************************************************)

(***** Coefficients of recurrence relation *****)

recuA[nu_,mu_,k_,g_]:=
  -g (((nu-mu+2*k-1) (nu-mu+2*k))/((2*nu+4*k-3) (2*nu+4*k-1)))

recuB[nu_,mu_,k_,g_]:=
  (nu+2*k)*(nu+2*k+1)-
  2 g (((nu+2*k) (nu+2*k+1)+mu^2-1)/((2*nu+4*k-1) (2*nu+4*k+3)))

recuC[nu_,mu_,k_,g_]:=
  -g (((mu+nu+2*k+1) (mu+nu+2*k+2))/((2*nu+4*k+3) (2*nu+4*k+5)))

(***** Size of Eigenmatrix *****)

matSizeSph[nu_,mu_,g_]:=
  10+Round[Sqrt[Abs[nu]+Abs[mu]+Abs[g]]];

(***** Eigenmatrix *****)

spheroidalMat[n_,m_,g_,matMax_]:=
  Module[{kMin},
    kMin=Round[Max[(m-n+Mod[n-m,2])/2,-matMax]];
    Table[Switch[j-k,-1,recuC[n,m,j,g],0,recuB[n,m,j,g],1,recuA[n,m,j,g],_,0],
    {j,kMin,matMax},{k,kMin,matMax}]]/;
  (intNQ[m] && intNQ[n] && 0<=m<=n)

spheroidalMat[n_,m_,g_,matMax_]:=
  Module[{kMin,kMax},
    kMin=Round[Max[(-m-n+Mod[n-m,2])/2,-matMax]];
    kMax=Round[Min[(m-n-2+Mod[n-m,2])/2,matMax]];
    Table[Switch[j-k,-1,recuC[n,m,j,g],0,recuB[n,m,j,g],1,recuA[n,m,j,g],_,0],
    {j,kMin,kMax},{k,kMin,kMax}]]/;
  (intNQ[m] && intNQ[n] && m>=0 && -m-1<n<m)

spheroidalMat[nu_,mu_,g_,matMax_]:=
  Table[Switch[j-k,-1,recuC[nu,mu,j,g],0,recuB[nu,mu,j,g],1,recuA[nu,mu,j,g],_,0],
  {j,-matMax,matMax},{k,-matMax,matMax}];

(***** Spheroidal eigenvalue : lambda[nu,mu,gamma] *****)

spheroidEigenSqrtG[n_,m_,g_]:=
  Module[{matMax,mat},
    matMax=matSizeSph[n,m,2*Sqrt[g]];
    mat=spheroidalMat[n,m,g,matMax];
    Sort[Eigenvalues[N[mat]]][[-matMax-1]]]/;
  (intNQ[m] && intNQ[n] && 0<=m<=n)

spheroidEigenSqrtG[n_,m_,g_]:=
  Module[{matMax,mat,pos},
    matMax=matSizeSph[n,m,2*Sqrt[g]];
    mat=spheroidalMat[n,m,g,matMax];
    pos=If[2*matMax+1<m,-matMax-1,n+1];
    Sort[Eigenvalues[N[mat]]][[pos]]]/;
  (intNQ[m] && intNQ[n] && 0<=n<m)

spheroidEigenSqrtG[n_,m_,g_]:=
  spheroidEigenSqrtG[-n-1,m,g]/;
  (intNQ[m] && intNQ[n] && m>=0 && n<0)

spheroidEigenSqrtG[n_,m_,g_]:=
  spheroidEigenSqrtG[n,-m,g]/;
  (intNQ[m] && intNQ[n] && m<0)

spheroidEigenSqrtG[nu_,mu_,g_]:=
  Module[{matMax,prec,nup,mup,gp,mat,pos},
    matMax=matSizeSph[nu,mu,2*Sqrt[g]];
    prec=If[Abs[Mod[nu,1]-0.5]<0.1,18,16];
    {nup,mup,gp}=SetPrecision[{nu,mu,g},prec];
    mat=spheroidalMat[nup,mup,gp,matMax];
    pos=First[Flatten[Position[
        Sort[Table[(nu+2*k)*(nu+2*k+1),{k,-matMax,matMax}]],nu*(nu+1)]]];
    Sort[Eigenvalues[N[mat]]][[pos]]]/;
  (Not[intNQ[mu]] || Not[intNQ[nu]])

SpheroidalEigenvalueLambda[nu_,mu_,g_]:=
  spheroidEigenSqrtG[nu,mu,g^2]/;
  (N[Mod[nu,1]]=!=0.5 && Precision[g]<Infinity)



(**************************************************************)
(*                        SetPrecision                        *)
(**************************************************************)

setPrecSph[x_]:=SetPrecision[x*10^18,100]/(10^18);

setPrecSph[x_,prec_]:=SetPrecision[x*10^prec,100]/(10^prec);



(**************************************************************)
(*               FlammerEigenvalueLambda[m,n,c]               *)
(**************************************************************)

(***** Coefficients of recurrence relation *****)

cfA[nu_,mu_,k_,g_]:=
  recuC[nu,mu,k-1,g]*recuA[nu,mu,k,g];

cfB[nu_,mu_,k_,g_]:=recuB[nu,mu,k,g];

gammaTerm[m_,r_,c_]:=
  (m+r)*(m+r+1)+(1-(4 m^2-1)/(2 m+2 r-1)/(2 m+2 r+3))/2*c;

betaTerm[m_,r_,c_]:=
  c^2*r*(r-1)*(2 m+r)*(2 m+r-1)/((2 m+2 r-1)^2 (2 m+2 r-3) (2 m+2 r+1));

(***** Seed functions of Flammer eigenvalue *****)

seedValueC[m_,n_,c_]:=c+spheroidEigenSqrtG[n,m,c];

(***** Method of Bouwkamp's approximation *****)

continuedFractionU1[m_,n_,c_,Lmn_]:=
  Module[{r,t,x},t=x+Lmn;
    gammaTerm[m,n-m,c]-Lmn+
    (Do[t=-betaTerm[m,r,c]/(gammaTerm[m,r-2,c]-Lmn+t),
    {r,If[EvenQ[n-m],0,1],n-m,2}];t)/.(x->0)];

continuedFractionU2[m_,n_,smax_,c_,Lmn_]:=
  Module[{r,t,x,evenmax},t=x+Lmn;
    evenmax=If[EvenQ[smax],smax,smax+1];
    Do[t=-betaTerm[m,n-m-r,c]/(gammaTerm[m,n-m-r,c]-Lmn+t),
    {r,-evenmax,-2,2}];t/.(x->0)]/;(smax>=3)

seriesDelta1[m_,n_,c_,Lmn_]:=
  Module[{s=0,so=1,k1=(n-m),g1},g1=1;
    While[k1>If[EvenQ[n-m],-2,-1],so=s;s+=g1;
     g1*=betaTerm[m,k1,c]/(continuedFractionU1[m,m+k1-2,c,Lmn]^2);
     k1-=2];s];

seriesDelta2[m_,n_,smax_,c_,Lmn_]:=
  Module[{s=0,so=1,k1=4,g1,evenmax},
    g1=(continuedFractionU2[m,n,smax,c,Lmn]^2)/betaTerm[m,n-m+2,c];
    evenmax=If[EvenQ[smax],smax+4,smax+5];
    While[k1<evenmax,so=s;s+=g1;
     g1*=(continuedFractionU2[m,n+k1-2,smax,c,Lmn]^2)/betaTerm[m,n-m+k1,c];
     k1+=2];s];

smaxSize[m_,n_]:=
  Which[Abs[n-m]<20,Floor[15+Abs[n-m]],Abs[n-m]>=20,35];

approximateBouwkamp[m_,n_,smax_,c_]:=
  Module[{Lmn},
  FixedPoint[
    Function[Lmn,
    Lmn+(continuedFractionU1[m,n,c,Lmn]+continuedFractionU2[m,n,smax,c,Lmn])/
    (seriesDelta1[m,n,c,Lmn]+seriesDelta2[m,n,smax,c,Lmn])],
   seedValueC[m,n,c],Floor[4/5*smax]]];

FlammerEigenvalueLambda[m_,n_,c_]:=
  If[m>n,0,
   If[Abs[Im[c^2]]>0,
    approximateBouwkamp[m,n,smaxSize[m,n],setPrecSph[c]^2],
    Re[approximateBouwkamp[m,n,smaxSize[m,n],setPrecSph[c]^2]]]]/;
    (Precision[c]<Infinity)



(**************************************************************)
(*            extraFlammerEigenvalueLambda[m,n,c]             *)
(**************************************************************)

(***** Size of Eigenmatrix *****)

extraMatSizeSph[nu_,mu_,g_]:=
  15+Round[Sqrt[Abs[nu]+Abs[mu]+Abs[g]]];

(***** Seed functions of Flammer eigenvalue *****)

seedEigenvalue[n_,m_,g_]:=
  Module[{matMax,mat},
    matMax=extraMatSizeSph[n,m,g];
    mat=spheroidalMat[n,m,g,matMax];
    Sort[Eigenvalues[N[mat]]][[-matMax-1]]]/;
  (intNQ[m] && intNQ[n] && 0<=m<=n)

seedEigenvalue[n_,m_,g_]:=
  Module[{matMax,mat,pos},
    matMax=extraMatSizeSph[n,m,g];
    mat=spheroidalMat[n,m,g,matMax];
    pos=If[2*matMax+1<m,-matMax-1,n+1];
    Sort[Eigenvalues[N[mat]]][[pos]]]/;
  (intNQ[m] && intNQ[n] && 0<=n<m)

seedEigenvalue[n_,m_,g_]:=
  seedEigenvalue[-n-1,m,g]/;
  (intNQ[m] && intNQ[n] && m>=0 && n<0)

seedEigenvalue[n_,m_,g_]:=
  seedEigenvalue[n,-m,g]/;
  (intNQ[m] && intNQ[n] && m<0)

extraSeedValueC[m_,n_,c_]:=c+seedEigenvalue[n,m,c];

(***** Method of Bouwkamp's approximation *****)

extraSMaxSize[m_,n_]:=Floor[30+Abs[n-m]];

extraBouwkamp[m_,n_,smax_,c_]:=
  Module[{Lmn},
  FixedPoint[
    Function[Lmn,
    Lmn+(continuedFractionU1[m,n,c,Lmn]+continuedFractionU2[m,n,smax,c,Lmn])/
    (seriesDelta1[m,n,c,Lmn]+seriesDelta2[m,n,smax,c,Lmn])],
   extraSeedValueC[m,n,c],smax]];

extraFlammerEigenvalueLambda[m_,n_,c_]:=
  If[m>n,0,
   If[Abs[Im[c^2]]>0,
    extraBouwkamp[m,n,extraSMaxSize[m,n],setPrecSph[c,50]^2],
    Re[extraBouwkamp[m,n,extraSMaxSize[m,n],setPrecSph[c,50]^2]]]]/;
    (Precision[c]<Infinity)



(**************************************************************)
(*                FlammerCoefficientD[m,n,c,r]                *)
(**************************************************************)

fcl[m_,n_,c_]:=fcl[m,n,c]=
  extraFlammerEigenvalueLambda[m,n,c];

coeffNGammaTerm[m_,c_,r_]:=
  (m+r)*(m+r+1)+c^2/2*(1-(4 m^2-1)/((2 m+2 r-1) (2 m+2 r+3)));

coeffNBetaTerm[m_,c_,r_]:=
  c^4*r*(r-1)*(2 m+r)*(2 m+r-1)/((2 m+2 r-1)^2 (2 m+2 r-3) (2 m+2 r+1));

coeffN[m_,n_,c_,2]:=coeffNGammaTerm[m,c,0]-fcl[m,n,c];

coeffN[m_,n_,c_,3]:=coeffNGammaTerm[m,c,1]-fcl[m,n,c];

coeffN[m_,n_,c_,r_]:=
  Module[{k,t,x},t=x;
  (Do[t=coeffNBetaTerm[m,c,k]/(coeffNGammaTerm[m,c,k]-fcl[m,n,c]-t),
  {k,r+50,r,-2}];t)/.(x->0)]/;(EvenQ[n-m]===EvenQ[r] && r>=4)

coeffN[m_,n_,c_,r_]:=0/;(EvenQ[n-m]=!=EvenQ[r])

coeffE[m_,n_,c_,r_]:=
  -(2 m+2 r-1)*(2 m+2 r+1)/((2 m+r) (2 m+r-1) c^2)*
  coeffN[m,n,c,r]/;(EvenQ[n-m]===EvenQ[r] && r>=2)

coefficientE[m_,n_,c_,r_]:=coefficientE[m,n,c,r]=coeffE[m,n,c,r];

cdEvenFunction[m_,n_,c_]:=
  Module[{s=0,so=1,k1=0,g1,g2,g3,zf},
  g1=Gamma[2 m+1];g2=1;g3=Gamma[m+1];zf=1;
    While[so!=s,so=s;s+=(g1/g2/g3*zf);
      g1*=((2 m+k1+1) (2 m+k1+2));
      g2*=(k1/2+1);g3*=(m+k1/2+1);
      zf*=(-coefficientE[m,n,c,k1+2]/4);k1+=2];s];

coefficientD[m_,n_,c_,0]:=
  (-1)^((n-m)/2)*Gamma[n+m+1]/
  (2^(n-m) Gamma[(n-m+2)/2] Gamma[(n+m+2)/2]*cdEvenFunction[m,n,c])/;
  (EvenQ[n-m]===True)

cdOddFunction[m_,n_,c_]:=
  Module[{s=0,so=1,k1=1,g1,g2,g3,zf},
  zf=1/2;g1=Gamma[2 m+3];g2=1;g3=Gamma[m+2];
    While[so!=s,so=s;s+=(g1/g2/g3*zf);
      g1*=((2 m+k1+2) (2 m+k1+3));
      g2*=((k1+1)/2);g3*=(m+(k1+3)/2);
      zf*=(-coefficientE[m,n,c,k1+2]/4);k1+=2];s];

coefficientD[m_,n_,c_,1]:=
  (-1)^((n-m-1)/2) Gamma[n+m+2]/
  (2^(n-m) Gamma[(n-m+1)/2] Gamma[(n+m+3)/2]*cdOddFunction[m,n,c])/;
  (OddQ[n-m]===True)

coefficientD[m_,n_,c_,r_]:=
  Module[{k},Product[coefficientE[m,n,c,k],{k,Mod[r,2]+2,r,2}]]*
  coefficientD[m,n,c,Mod[r,2]]/;
  (EvenQ[n-m]===EvenQ[r] && r>=2 && Chop[c]=!=0)

coefficientD[m_,n_,c_,r_]:=
  0/;(EvenQ[n-m]=!=EvenQ[r] || (Chop[n-m-r]=!=0 && Chop[c]===0))

coefficientD[m_,n_,c_,r_]:=1/;(Chop[n-m-r]===0 && Chop[c]===0)

negativeTermA[m_,c_,r_]:=
  c^2*(2 m+r)*(2 m+r-1)/((2 m+2 r-1) (2 m+2 r+1));

negativeTermB[m_,n_,c_,r_]:=
  (m+r)*(m+r+1)-fcl[m,n,c]+c^2*(2 (m+r) (m+r+1)-2 m^2-1)/
  ((2 m+2 r-1) (2 m+2 r+3));

negativeTermC[m_,c_,r_]:=
  c^2*(r+1)*(r+2)/((2 m+2 r+1) (2 m+2 r+3));

coeffF[m_,n_,c_,r_]:=
  Module[{k,t,x},t=x;
  (Do[t=-negativeTermA[m,c,k+2]/
  (negativeTermB[m,n,c,k]+negativeTermC[m,c,k-2]*t),
  {k,r-50,r,2}];t)/.(x->0)]/;(r<=-1)

coefficientF[m_,n_,c_,r_]:=coefficientF[m,n,c,r]=coeffF[m,n,c,r];

coefficientD[m_,n_,c_,r_]:=
  Module[{k},Product[coefficientF[m,n,c,k],{k,Mod[r,2]-2,r,-2}]]*
  coefficientD[m,n,c,Mod[r,2]]/;
  (EvenQ[n-m]===EvenQ[r] && r<=-1 && Chop[c]=!=0)

FlammerCoefficientD[m_Integer,n_Integer,c_,r_Integer]:=
  FlammerCoefficientD[m,n,c,r]=
  coefficientD[m,n,setPrecSph[c,30],r];



(**************************************************************)
(*              FlammerCoefficientDRho[m,n,c,r]               *)
(**************************************************************)

coeffG[m_,n_,c_,r_]:=
  -negativeTermA[m,c,-r]/
  (negativeTermB[m,n,c,-r-2]+negativeTermC[m,c,-r-4] coeffG[m,n,c,r+2])/;
  (EvenQ[n-m]===EvenQ[r] && N[2 m+1]<=r<=N[2 m+50])

coeffG[m_,n_,c_,r_]:=
  -negativeTermA[m,c,-r]/negativeTermB[m,n,c,-r-2]/;
  (EvenQ[n-m]===EvenQ[r] && r>N[2 m+50])

coefficientG[m_,n_,c_,r_]:=coefficientG[m,n,c,r]=coeffG[m,n,c,r];

continuedFractionRho[m_,n_,c_,r_]:=
  1/(negativeTermB[m,n,c,-r]-negativeTermC[m,c,-r-2]*
  negativeTermA[m,c,-r]*continuedFractionRho[m,n,c,r+2])/;
  (EvenQ[n-m]===EvenQ[r] && N[2 m+1]<=r<=N[2 m+50])

continuedFractionRho[m_,n_,c_,r_]:=
  1/negativeTermB[m,n,c,-r]/;(EvenQ[n-m]===EvenQ[r] && r>N[2 m+50])

coefficientDRho[m_,n_,c_,r_]:=
  c^2*FlammerCoefficientD[m,n,c,-2 m]*continuedFractionRho[m,n,c,r]/
  ((2 m+1) (2 m-1))/;(EvenQ[n-m]===True && N[r]===N[2 m+2] && Chop[c]=!=0)

coefficientDRho[m_,n_,c_,r_]:=
  -c^2*FlammerCoefficientD[m,n,c,-2 m+1]*continuedFractionRho[m,n,c,r]/
  ((2 m-1) (2 m-3))/;(OddQ[n-m]===True && N[r]===N[2 m+1] && Chop[c]=!=0)

coefficientDRho[m_,n_,c_,r_]:=
  Module[{k,rif=If[EvenQ[r]===True,2 m+2,2 m+1]},
  Product[coefficientG[m,n,c,k],{k,rif,r-2,2}]*
  coefficientDRho[m,n,c,rif]]/;
  (EvenQ[n-m]===EvenQ[r] && N[r]>=N[2 m+3] && Chop[c]=!=0)

coefficientDRho[m_,n_,c_,r_]:=0/;(EvenQ[n-m]=!=EvenQ[r] || Chop[c]===0)

FlammerCoefficientDRho[m_Integer,n_Integer,c_,r_Integer]:=
  FlammerCoefficientDRho[m,n,c,r]=
  coefficientDRho[m,n,setPrecSph[c,30],r];



(**************************************************************)
(*                 spherical Bessel functions                 *)
(**************************************************************)

(*
initialBessel[n_]:=
  10^-100/;(Chop[n]===0)

initialBessel[n_]:=
  (Gamma[2 n+2]/(2^n Gamma[n+1] 10^30))^(1/n)/;
  (Chop[n]=!=0)

sphericalBesselJ[n_,z_]:=
  2^n Gamma[n+1]/Gamma[2 n+2] z^n/;
  (Abs[z]<=N[initialBessel[n]])

sphericalBesselJ[n_,z_]:=
  BesselJ[n+1/2,z] Sqrt[Pi/(2 z)]/;
  (Re[z]>=0 && Abs[z]>N[initialBessel[n]])

sphericalBesselJ[n_,z_]:=
  (-1)^n sphericalBesselJ[n,-z]/;
  (Re[z]<0 && Abs[z]>N[initialBessel[n]])

sphericalBesselY[n_,z_]:=
  -Gamma[2 n+1]/(2^n Gamma[n+1] z^(n+1))/;
  (Abs[z]<=N[initialBessel[n]])

sphericalBesselY[n_,z_]:=
  BesselY[n+1/2,z] Sqrt[Pi/(2 z)]/;
  (Re[z]>=0 && Abs[z]>N[initialBessel[n]])

sphericalBesselY[n_,z_]:=
  (-1)^(n+1) sphericalBesselY[n,-z]/;
  (Re[z]<0 && Abs[z]>N[initialBessel[n]])

sphericalHankelH1[n_,z_]:=
  sphericalBesselJ[n,z]+I sphericalBesselY[n,z];

sphericalHankelH2[n_,z_]:=
  sphericalBesselJ[n,z]-I sphericalBesselY[n,z];
*)


(**************************************************************)
(*                      summationMaxf[c]                      *)
(**************************************************************)

summationMaxf[c_]:=
  Floor[7.57+14.14 Abs[c]^(1/3)+1.3 Sqrt[Abs[c]]+
  0.45 Abs[c]+0.14 Log[Abs[c]]];



(**************************************************************)
(*                   radialFactorRho[m,n,c]                   *)
(**************************************************************)

factorRho[m_,n_,c_]:=
  Module[{s=0,so=1,k1=If[EvenQ[n-m],0,1],g1,g2},
    g1=Gamma[2 m+k1+1];g2=Gamma[k1+1];
	While[k1<summationMaxf[c],so=s; 
	s+=(g1/g2*FlammerCoefficientD[m,n,c,k1]);
	g1*=((2 m+k1+1) (2 m+k1+2));
	g2*=((k1+1) (k1+2)); k1+=2];s]

radialFactorRho[m_,n_,c_]:=
  radialFactorRho[m,n,c]=factorRho[m,n,setPrecSph[c,30]];



(**************************************************************)
(*                 JoiningFactorKappa1[m,n,c]                 *)
(**************************************************************)

JoiningFactorKappa1[m_,n_,c_]:=
  factorKappa1[m,n,setPrecSph[c,30]];

factorKappa1[m_,n_,c_]:=
  factorRho[m,n,c]*(2 m+1)*Gamma[m+n+1]/
  (2^(m+n)*c^m*Gamma[m+1]*Gamma[(n-m)/2+1]*Gamma[(n+m)/2+1]*
  FlammerCoefficientD[m,n,c,0])/;(EvenQ[n-m]===True)

factorKappa1[m_,n_,c_]:=
  factorRho[m,n,c]*(2 m+3)*Gamma[m+n+2]/
  (2^(m+n)*c^(m+1)*Gamma[m+1]*Gamma[(n-m+1)/2]*Gamma[(n+m+3)/2]*
  FlammerCoefficientD[m,n,c,1])/;(OddQ[n-m]===True)



(**************************************************************)
(*                 JoiningFactorKappa2[m,n,c]                 *)
(**************************************************************)

JoiningFactorKappa2[m_,n_,c_]:=
  factorKappa2[m,n,setPrecSph[c,30]]

factorKappa2[m_,n_,c_]:=
  factorRho[m,n,c]*2^(n-m)*Gamma[2 m+1]*Gamma[(n-m)/2+1]*
  Gamma[(n+m)/2+1]*FlammerCoefficientD[m,n,c,-2 m]/
  ((2 m-1)*c^(m-1)*Gamma[m+1]*Gamma[m+n+1])/;
  (EvenQ[n-m]===True)

factorKappa2[m_,n_,c_]:=
  -factorRho[m,n,c]*2^(n-m)*Gamma[2 m+1]*Gamma[(n-m+1)/2]*
  Gamma[(n+m+3)/2]*FlammerCoefficientD[m,n,c,-2 m+1]/
  ((2 m-3)*(2 m-1)*c^(m-2)*Gamma[m+1]*Gamma[m+n+2])/;
  (OddQ[n-m]===True)



(**************************************************************)
(*                          unitDisk                          *)
(**************************************************************)

unitDiskFerrers[m_,z_]:=(1-z^2)^(m/2)/z^m;

unitDiskHobson[m_,z_]:=(1-1/(z^2))^(m/2);



(**************************************************************)
(*                    legenPTermF[m,n,c,z]                    *)
(**************************************************************)

legenPTermF[m_,n_,c_,z_]:=
  Module[{k1,rmin=If[EvenQ[n-m],-2 m,-2 m+1],rmax=If[EvenQ[n-m],-2,-1]},
  Sum[FlammerCoefficientD[m,n,c,k1]*
	LegendreP[m+k1,m,3,z],{k1,rmin,rmax,2}]]/;(m>0)

legenPTermF[m_,n_,c_,z_]:=0/;(N[m]===0.)



(**************************************************************)
(*                      convergRadius[c]                      *)
(**************************************************************)

convergRadius[c_]:=
  If[Abs[c]<0.5,2,If[Abs[c]>14,1.3,(547-14*Abs[c])/270]];



(*---------------------------------------------------------------------------*)
(*                                                                           *)
(*                                                                           *)
(*                         Spheroidal wave functions                         *)
(*                             (Flammer scheme)                              *)
(*                                                                           *)
(*                                                                           *)
(*---------------------------------------------------------------------------*)

(**************************************************************)
(*                   FlammerWaveS1[m,n,c,z]                   *)
(**************************************************************)

flammerS1legP[m_,n_,c_,a_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  Module[{s=0,so=1,k1=If[EvenQ[n-m],0,1]},
	While[(so!=s && k1<(summationMaxf[c]+10)),so=s; 
	s+=(FlammerCoefficientD[m,n,spC,k1]*
	LegendreP[m+k1,m,a,spZ]); k1+=2];s]];

FlammerWaveS1[m_,n_,c_,z_]:=
  FlammerWaveS1[m,n,c,2,z];

FlammerWaveS1[m_,n_,c_,a_,z_]:=
  (-1)^m*LegendreP[m,n,a,z]/;(N[Abs[c]]===0.)

FlammerWaveS1[m_,n_,c_,1,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  (-1)^m*flammerS1legP[m,n,spC,1,spZ]]/;(N[Abs[c]]=!=0.)

FlammerWaveS1[m_,n_,c_,2,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  (-1)^m*flammerS1legP[m,n,spC,2,spZ]]/;
  (Abs[z]<=convergRadius[c] && N[Abs[c]]=!=0.)

FlammerWaveS1[m_,n_,c_,2,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  Module[{s=0,so=1,k1=0,g1,g2,if},
  if=I^(k1+m-n);g1=Gamma[2 m+k1+1];g2=Gamma[k1+1];
	While[(so!=s && k1<summationMaxf[c]),so=s; 
	s+=(g1/g2*if*FlammerCoefficientD[m,n,spC,k1]*
	SphericalBesselJ[m+k1,spC*spZ]);
	g1*=((2 m+k1+1) (2 m+k1+2));g2*=((k1+1) (k1+2));
	if*=(-1); k1+=2];s] unitDiskFerrers[m,spZ]*
  (2 m+1) Gamma[n+m+1]/(2^(n+m) spC^m Gamma[m+1]*
  Gamma[(n-m)/2+1] Gamma[(n+m)/2+1]*
  FlammerCoefficientD[m,n,spC,0])]/;
  (Abs[z]>convergRadius[c] && EvenQ[n-m] && N[Abs[c]]=!=0.)

FlammerWaveS1[m_,n_,c_,2,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  Module[{s=0,so=1,k1=1,g1,g2,if},
  if=I^(k1+m-n);g1=Gamma[2 m+k1+1];g2=Gamma[k1+1];
	While[(so!=s && k1<summationMaxf[c]),so=s; 
	s+=(g1/g2*if*FlammerCoefficientD[m,n,spC,k1]*
	SphericalBesselJ[m+k1,spC*spZ]);
	g1*=((2 m+k1+1) (2 m+k1+2));g2*=((k1+1) (k1+2));
	if*=(-1); k1+=2];s] unitDiskFerrers[m,spZ]*
  (2 m+3) Gamma[n+m+2]/(2^(n+m) spC^(m+1) Gamma[m+1]*
  Gamma[(n-m+1)/2] Gamma[(n+m+3)/2]*
  FlammerCoefficientD[m,n,spC,1])]/;
  (Abs[z]>convergRadius[c] && OddQ[n-m] && N[Abs[c]]=!=0.)

FlammerWaveS1[m_,n_,c_,3,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  flammerS1legP[m,n,spC,3,spZ]]/;
  (Abs[z]<=convergRadius[c] && N[Abs[c]]=!=0.)

FlammerWaveS1[m_,n_,c_,3,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  Module[{s=0,so=1,k1=0,g1,g2,if},
  if=I^(k1+m-n);g1=Gamma[2 m+k1+1];g2=Gamma[k1+1];
	While[(so!=s && k1<summationMaxf[c]),so=s; 
	s+=(g1/g2*if*FlammerCoefficientD[m,n,spC,k1]*
	SphericalBesselJ[m+k1,spC*spZ]);
	g1*=((2 m+k1+1) (2 m+k1+2));g2*=((k1+1) (k1+2));
	if*=(-1); k1+=2];s] unitDiskHobson[m,spZ]*
  (2 m+1) Gamma[n+m+1]/(2^(n+m) spC^m Gamma[m+1]*
  Gamma[(n-m)/2+1] Gamma[(n+m)/2+1]*
  FlammerCoefficientD[m,n,spC,0])]/;
  (Abs[z]>convergRadius[c] && EvenQ[n-m] && N[Abs[c]]=!=0.)

FlammerWaveS1[m_,n_,c_,3,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  Module[{s=0,so=1,k1=1,g1,g2,if},
  if=I^(k1+m-n);g1=Gamma[2 m+k1+1];g2=Gamma[k1+1];
	While[(so!=s && k1<summationMaxf[c]),so=s; 
	s+=(g1/g2*if*FlammerCoefficientD[m,n,spC,k1]*
	SphericalBesselJ[m+k1,spC*spZ]);
	g1*=((2 m+k1+1) (2 m+k1+2));g2*=((k1+1) (k1+2));
	if*=(-1); k1+=2];s] unitDiskHobson[m,spZ]*
  (2 m+3) Gamma[n+m+2]/(2^(n+m) spC^(m+1) Gamma[m+1]*
  Gamma[(n-m+1)/2] Gamma[(n+m+3)/2]*
  FlammerCoefficientD[m,n,spC,1])]/;
  (Abs[z]>convergRadius[c] && OddQ[n-m] && N[Abs[c]]=!=0.)

(**************************************************************)
(*                   FlammerWaveS2[m,n,c,z]                   *)
(**************************************************************)

flammerS2legQ[m_,n_,c_,a_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  Module[{s=0,so=1,k1=If[EvenQ[n-m],-2 m,-2 m+1]},
	While[(so!=s && k1<(summationMaxf[c]+40)),so=s; 
	s+=(FlammerCoefficientD[m,n,spC,k1]*
	LegendreQ[m+k1,m,a,spZ]); k1+=2];s]];

flammerS2legP[m_,n_,c_,a_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  Module[{s=0,so=1,k1=If[EvenQ[n-m],2 m+2,2 m+1]},
	While[(so!=s && k1<(summationMaxf[c]+40)),so=s; 
	s+=(FlammerCoefficientDRho[m,n,spC,k1]*
	LegendreP[k1-m-1,m,a,spZ]); k1+=2];s]];

FlammerWaveS2[m_,n_,c_,z_]:=
  FlammerWaveS2[m,n,c,2,z];

FlammerWaveS2[m_,n_,c_,a_,z_]:=
  (-1)^m*LegendreQ[m,n,a,z]/;(N[Abs[c]]===0.)

FlammerWaveS2[m_,n_,c_,1,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  (-1)^m*(flammerS2legQ[m,n,spC,1,spZ]+flammerS2legP[m,n,spC,1,spZ])]/;
  (N[Abs[c]]=!=0.)

FlammerWaveS2[m_,n_,c_,2,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  (-1)^m*(flammerS2legQ[m,n,spC,2,spZ]+flammerS2legP[m,n,spC,2,spZ])]/;
  (Abs[z]<=convergRadius[c] && N[Abs[c]]=!=0.)

FlammerWaveS2[m_,n_,c_,2,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  ((1-spZ)^(m/2)/(spZ-1)^(m/2))*
  JoiningFactorKappa2[m,n,spC]*FlammerWaveR2[m,n,spC,spZ]+
  Pi/2*((1-spZ)^((m-1)/2)/(spZ-1)^((m-1)/2))*
  JoiningFactorKappa1[m,n,spC]*FlammerWaveR1[m,n,spC,spZ]+
  Pi/2*(-1)^m*(Sqrt[spZ-1]/Sqrt[1-spZ])*legenPTermF[m,n,spC,spZ]]/;
  (Abs[z]>convergRadius[c] && N[Abs[c]]=!=0.)

FlammerWaveS2[m_,n_,c_,3,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  flammerS2legQ[m,n,spC,3,spZ]+flammerS2legP[m,n,spC,3,spZ]]/;
  (Abs[z]<=convergRadius[c] && N[Abs[c]]=!=0.)

FlammerWaveS2[m_,n_,c_,3,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  Module[{s=0,so=1,k1=0,g1,g2,if},
  if=I^(k1+m-n);g1=Gamma[2 m+k1+1];g2=Gamma[k1+1];
	While[(so!=s && k1<(summationMaxf[c]+40)),so=s; 
	s+=(g1/g2*if*FlammerCoefficientD[m,n,spC,k1]*
	SphericalBesselY[m+k1,spC*spZ]);
	g1*=((2 m+k1+1) (2 m+k1+2));g2*=((k1+1) (k1+2));
	if*=(-1); k1+=2];s] unitDiskHobson[m,spZ]*
	(2^(n-m) Gamma[2 m+1] Gamma[(n-m)/2+1]*
	Gamma[(n+m)/2+1] FlammerCoefficientD[m,n,spC,-2 m])/
	(spC^(m-1) (2 m-1) Gamma[m+1] Gamma[n+m+1])]/;
  (Abs[z]>convergRadius[c] && EvenQ[n-m] && N[Abs[c]]=!=0.)

FlammerWaveS2[m_,n_,c_,3,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  Module[{s=0,so=1,k1=1,g1,g2,if},
  if=I^(k1+m-n);g1=Gamma[2 m+k1+1];g2=Gamma[k1+1];
	While[(so!=s && k1<(summationMaxf[c]+40)),so=s; 
	s+=(g1/g2*if*FlammerCoefficientD[m,n,spC,k1]*
	SphericalBesselY[m+k1,spC*spZ]);
	g1*=((2 m+k1+1) (2 m+k1+2));g2*=((k1+1) (k1+2));
	if*=(-1); k1+=2];s] (-unitDiskHobson[m,spZ])*
	(2^(n-m) Gamma[2 m+1] Gamma[(n-m+1)/2]*
	Gamma[(n+m+3)/2] FlammerCoefficientD[m,n,spC,-2 m+1])/
	(spC^(m-2) (2 m-3) (2 m-1) Gamma[m+1] Gamma[n+m+2])]/;
  (Abs[z]>convergRadius[c] && OddQ[n-m] && N[Abs[c]]=!=0.)

(**************************************************************)
(*                   FlammerWaveR1[m,n,c,z]                   *)
(**************************************************************)

FlammerWaveR1[m_,n_,c_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  FlammerWaveS1[m,n,spC,3,spZ]*
  (2^(n+m) spC^m Gamma[m+1] Gamma[(n-m)/2+1]*
  Gamma[(n+m)/2+1] FlammerCoefficientD[m,n,spC,0])/
  ((2 m+1) Gamma[n+m+1])/radialFactorRho[m,n,spC]]/;
  (Abs[z]<=convergRadius[c] && EvenQ[n-m])

FlammerWaveR1[m_,n_,c_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  FlammerWaveS1[m,n,spC,3,spZ]*
  (2^(n+m) spC^(m+1) Gamma[m+1] Gamma[(n-m+1)/2]*
  Gamma[(n+m+3)/2] FlammerCoefficientD[m,n,spC,1])/
  ((2 m+3) Gamma[n+m+2])/radialFactorRho[m,n,spC]]/;
  (Abs[z]<=convergRadius[c] && OddQ[n-m])

FlammerWaveR1[m_,n_,c_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  Module[{s=0,so=1,k1=If[EvenQ[n-m],0,1],g1,g2,if},
  if=I^(k1+m-n);g1=Gamma[2 m+k1+1];g2=Gamma[k1+1];
	While[(so!=s && k1<summationMaxf[c]),so=s; 
	s+=(g1/g2*if*FlammerCoefficientD[m,n,spC,k1]*
	SphericalBesselJ[m+k1,spC*spZ]);
	g1*=((2 m+k1+1) (2 m+k1+2));g2*=((k1+1) (k1+2));
	if*=(-1); k1+=2];s]/radialFactorRho[m,n,spC]*
	unitDiskHobson[m,spZ]]/;(Abs[z]>convergRadius[c])

(**************************************************************)
(*                   FlammerWaveR2[m,n,c,z]                   *)
(**************************************************************)

FlammerWaveR2[m_,n_,c_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  FlammerWaveS2[m,n,spC,3,spZ]/(2^(n-m) Gamma[2 m+1]*
  Gamma[(n-m)/2+1] Gamma[(n+m)/2+1]*
  FlammerCoefficientD[m,n,spC,-2 m])*
  (spC^(m-1) (2 m-1) Gamma[m+1] Gamma[n+m+1])/
  radialFactorRho[m,n,spC]]/;
  (Abs[z]<=convergRadius[c] && EvenQ[n-m])

FlammerWaveR2[m_,n_,c_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  -FlammerWaveS2[m,n,spC,3,spZ]/(2^(n-m) Gamma[2 m+1]*
  Gamma[(n-m+1)/2] Gamma[(n+m+3)/2]*
  FlammerCoefficientD[m,n,spC,-2 m+1])*
  (spC^(m-2) (2 m-3) (2 m-1) Gamma[m+1] Gamma[n+m+2])/
  radialFactorRho[m,n,spC]]/;
  (Abs[z]<=convergRadius[c] && OddQ[n-m])

FlammerWaveR2[m_,n_,c_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  Module[{s=0,so=1,k1=If[EvenQ[n-m],0,1],g1,g2,if},
  if=I^(k1+m-n);g1=Gamma[2 m+k1+1];g2=Gamma[k1+1];
	While[(so!=s && k1<(summationMaxf[c]+40)),so=s; 
	s+=(g1/g2*if*FlammerCoefficientD[m,n,spC,k1]*
	SphericalBesselY[m+k1,spC*spZ]);
	g1*=((2 m+k1+1) (2 m+k1+2));g2*=((k1+1) (k1+2));
	if*=(-1); k1+=2];s]/radialFactorRho[m,n,spC]*
	unitDiskHobson[m,spZ]]/;(Abs[z]>convergRadius[c])

(**************************************************************)
(*                   FlammerWaveR3[m,n,c,z]                   *)
(**************************************************************)

FlammerWaveR3[m_,n_,c_,z_]:=
  FlammerWaveR1[m,n,c,z]+I FlammerWaveR2[m,n,c,z];

(**************************************************************)
(*                   FlammerWaveR4[m,n,c,z]                   *)
(**************************************************************)

FlammerWaveR4[m_,n_,c_,z_]:=
  FlammerWaveR1[m,n,c,z]-I FlammerWaveR2[m,n,c,z];



(*---------------------------------------------------------------------------*)
(*                                                                           *)
(*                                                                           *)
(*                         Spheroidal wave functions                         *)
(*                         (Meixner-Schafke scheme)                          *)
(*                                                                           *)
(*                                                                           *)
(*---------------------------------------------------------------------------*)

(**************************************************************)
(*                 FlammerNormFactorN[m,n,c]                  *)
(**************************************************************)

flammerNormFN[m_,n_,c_]:=
  Module[{s=0,so=1,k1=If[EvenQ[n-m],0,1],g1,g2},
    g1=Gamma[2 m+k1+1];g2=Gamma[k1+1];
	While[(so!=s && k1<summationMaxf[c]),so=s; 
	s+=(g1/g2/(2 m+2 k1+1)*FlammerCoefficientD[m,n,c,k1]^2);
	g1*=((2 m+k1+1) (2 m+k1+2));
	g2*=((k1+1) (k1+2)); k1+=2];
  2*s];

FlammerNormFactorN[m_,n_,c_]:=
  FlammerNormFactorN[m,n,c]=flammerNormFN[m,n,setPrecSph[c,30]];

(*****************************************************************************)
(*                         SpheroidalWavePS[n,m,g,z]                         *)
(*****************************************************************************)

SpheroidalWavePS[n_,m_,g_,z_]:=
  SpheroidalWavePS[n,m,2,g,z];

SpheroidalWavePS[n_,m_,a_,g_,z_]:=
  LegendreP[n,m,a,z]/;(Precision[z]<Infinity && N[Abs[g]]===0.)

SpheroidalWavePS[n_,m_,2,g_,z_]:=
  Module[{spG,spZ,ans},
    {spG,spZ}=setPrecSph[{g,z}];
    ans=(-1)^m*FlammerWaveS1[m,n,spG,2,spZ]*Sqrt[Abs[2/(2 n+1)*Gamma[n+m+1]/
      Gamma[n-m+1]/FlammerNormFactorN[m,n,spG]]];
    If[Abs[Im[z]]>0,ans,Chop[ans,10^-10]]]/;
  (Precision[z]<Infinity && N[Abs[g]]=!=0.)

SpheroidalWavePS[n_,m_,3,g_,z_]:=
  Module[{spG,spZ,ans},
    {spG,spZ}=setPrecSph[{g,z}];
    ans=FlammerWaveS1[m,n,spG,3,spZ]*Sqrt[Abs[2/(2 n+1)*Gamma[n+m+1]/
      Gamma[n-m+1]/FlammerNormFactorN[m,n,spG]]];
    If[Abs[Im[z]]>0,ans,Chop[ans,10^-10]]]/;
  (Precision[z]<Infinity && N[Abs[g]]=!=0.)

(*****************************************************************************)
(*                         SpheroidalWaveQS[n,m,g,z]                         *)
(*****************************************************************************)

SpheroidalWaveQS[n_,m_,g_,z_]:=
  SpheroidalWaveQS[n,m,2,g,z];

SpheroidalWaveQS[n_,m_,a_,g_,z_]:=
  LegendreQ[n,m,a,z]/;(Precision[z]<Infinity && N[Abs[g]]===0.)

SpheroidalWaveQS[n_,m_,2,g_,z_]:=
  Module[{spG,spZ,ans},
    {spG,spZ}=setPrecSph[{g,z}];
    ans=(-1)^m*FlammerWaveS2[m,n,spG,2,spZ]*Sqrt[Abs[2/(2 n+1)*Gamma[n+m+1]/
      Gamma[n-m+1]/FlammerNormFactorN[m,n,spG]]];
    If[Abs[Im[z]]>0,ans,Chop[ans,10^-10]]]/;
  (Precision[z]<Infinity && N[Abs[g]]=!=0.)

SpheroidalWaveQS[n_,m_,3,g_,z_]:=
  Module[{spG,spZ,ans},
    {spG,spZ}=setPrecSph[{g,z}];
    ans=FlammerWaveS2[m,n,spG,3,spZ]*Sqrt[Abs[2/(2 n+1)*Gamma[n+m+1]/
      Gamma[n-m+1]/FlammerNormFactorN[m,n,spG]]];
    If[Abs[Im[z]]>0,ans,Chop[ans,10^-10]]]/;
  (Precision[z]<Infinity && N[Abs[g]]=!=0.)

(*****************************************************************************)
(*                         SpheroidalWaveS1[n,m,g,z]                         *)
(*****************************************************************************)

SpheroidalWaveS1[n_,m_,g_,z_]:=
  If[Abs[Im[z]]>0,
    FlammerWaveR1[m,n,g,z],Chop[FlammerWaveR1[m,n,g,z],10^-10]]/;
  (Precision[z]<Infinity)

(*****************************************************************************)
(*                         SpheroidalWaveS2[n,m,g,z]                         *)
(*****************************************************************************)

SpheroidalWaveS2[n_,m_,g_,z_]:=
  If[Abs[Im[z]]>0,
    FlammerWaveR2[m,n,g,z],Chop[FlammerWaveR2[m,n,g,z],10^-10]]/;
  (Precision[z]<Infinity)

(*****************************************************************************)
(*                         SpheroidalWaveS3[n,m,g,z]                         *)
(*****************************************************************************)

SpheroidalWaveS3[n_,m_,g_,z_]:=
  If[Abs[Im[z]]>0,
    FlammerWaveR3[m,n,g,z],Chop[FlammerWaveR3[m,n,g,z],10^-10]]/;
  (Precision[z]<Infinity)

(*****************************************************************************)
(*                         SpheroidalWaveS4[n,m,g,z]                         *)
(*****************************************************************************)

SpheroidalWaveS4[n_,m_,g_,z_]:=
  If[Abs[Im[z]]>0,
    FlammerWaveR4[m,n,g,z],Chop[FlammerWaveR4[m,n,g,z],10^-10]]/;
  (Precision[z]<Infinity)



(*---------------------------------------------------------------------------*)
(*                                                                           *)
(*                                                                           *)
(*                         Spheroidal wave functions                         *)
(*                (Stratton-Morse-Chu-Little-Corbato scheme)                 *)
(*                                                                           *)
(*                                                                           *)
(*---------------------------------------------------------------------------*)

(**************************************************************)
(*                  StrattonWaveS1[m,n,c,z]                   *)
(**************************************************************)

StrattonWaveS1[m_,n_,c_,z_]:=
  StrattonWaveS1[m,n,c,2,z];

StrattonWaveS1[m_,n_,c_,a_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  FlammerWaveS1[m,n,spC,a,spZ] Gamma[n+m+1]/
  Gamma[n-m+1]/radialFactorRho[m,n,spC]];

(**************************************************************)
(*                  StrattonWaveS2[m,n,c,z]                   *)
(**************************************************************)

StrattonWaveS2[m_,n_,c_,z_]:=
  StrattonWaveS2[m,n,c,2,z];

StrattonWaveS2[m_,n_,c_,a_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  FlammerWaveS2[m,n,spC,a,spZ] Gamma[n+m+1]/
  Gamma[n-m+1]/radialFactorRho[m,n,spC]];



(*---------------------------------------------------------------------------*)
(*                                                                           *)
(*                                                                           *)
(*                     Spheroidal wave Cosine functions                      *)
(*                            (Flammer scheme)                               *)
(*                                                                           *)
(*                                                                           *)
(*---------------------------------------------------------------------------*)

(**************************************************************)
(*                  jfKappa1parKappa2[m,n,c]                  *)
(**************************************************************)

jfKappa1parKappa2[m_,n_,c_]:=
  jfKappa1parKappa2[m,n,c]=fKappa1Kappa2[m,n,setPrecSph[c,30]]

fKappa1Kappa2[m_,n_,c_]:=
  (4^m*(4 m^2-1)*Gamma[(1+m+n)/2]^2)/
  (c*Pi*FlammerCoefficientD[m,n,c,0]*FlammerCoefficientD[m,n,c,-2 m]*
  Gamma[2 m+1]*Gamma[(2-m+n)/2]^2)/;(EvenQ[n-m])

fKappa1Kappa2[m_,n_,c_]:=
  -(4^(m+1)*(2 m-3)*(2 m-1)*(2 m+3)*Gamma[(2+m+n)/2]^2)/
  (c^3*Pi*FlammerCoefficientD[m,n,c,1]*FlammerCoefficientD[m,n,c,-2 m+1]*
  Gamma[1+2 m]*Gamma[(1-m+n)/2]^2)/;(OddQ[n-m])

(**************************************************************)
(*                       fwsLegCos[m,z]                       *)
(**************************************************************)

fwsLegCos[m_,z_]:=
  If[OddQ[m] && Re[Sin[z]]>=0,-1,1];

(**************************************************************)
(*                      fwsDiskCos[m,z]                       *)
(**************************************************************)

fwsDiskCos[m_,z_]:=
  (-1)^(m/2)/;(Im[z]>0 || (N[Im[z]]===0. && Re[Sin[2 z]]<0))

fwsDiskCos[m_,z_]:=
  (-1)^(-m/2)/;(Im[z]<0 || (N[Im[z]]===0. && Re[Sin[2 z]]>=0))

(**************************************************************)
(*                 FlammerWaveS1Cos[m,n,c,z]                  *)
(**************************************************************)

FlammerWaveS1Cos[m_,n_,c_,z_]:=
  fwsLegCos[m,z]*LegendreP[n,m,2,Cos[z]]/;
  (Precision[z]<Infinity && N[Abs[c]]===0.)

FlammerWaveS1Cos[m_,n_,c_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  fwsDiskCos[m,spZ]*JoiningFactorKappa1[m,n,spC]*
  FlammerWaveR1[m,n,spC,Cos[spZ]]]/;
  (Precision[z]<Infinity && N[Abs[c]]=!=0.)

(**************************************************************)
(*                 FlammerWaveS2Cos[m,n,c,z]                  *)
(**************************************************************)

FlammerWaveS2Cos[m_,n_,c_,z_]:=
  If[(N[Im[z]]===0. && Re[Sin[z]]>0),Re[FlammerWaveS2[m,n,c,Cos[z]]],
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  fwsDiskCos[m,spZ]*JoiningFactorKappa2[m,n,spC]*
  FlammerWaveR2[m,n,spC,Cos[spZ]]+
  Pi/2*fwsDiskCos[m-1,spZ]*JoiningFactorKappa1[m,n,spC]*
  FlammerWaveR1[m,n,spC,Cos[spZ]]+
  Pi/2*(-1)^m*fwsDiskCos[-1,spZ]*legenPTermF[m,n,spC,Cos[spZ]]]]/;
  (Precision[z]<Infinity && N[Abs[c]]=!=0.)

(**************************************************************)
(*                 FlammerWaveR1Cos[m,n,c,z]                  *)
(**************************************************************)

FlammerWaveR1Cos[m_,n_,c_,z_]:=
  (-I*Tan[z])^m*((Cos[z]^2/(Cos[z]^2-1))^(m/2))*FlammerWaveR1[m,n,c,Cos[z]]/;
  (Precision[z]<Infinity)

(**************************************************************)
(*                 FlammerWaveR2Cos[m,n,c,z]                  *)
(**************************************************************)

FlammerWaveR2Cos[m_,n_,c_,z_,opts___?OptionQ]:=
  Module[{optf},
  optf=BranchCut/.{opts}/.Options[FlammerWaveR2Cos];
  If[optf==1,flammerWaveR2CosType1[m,n,c,z],
             flammerWaveR2CosType2[m,n,c,z]]]/;(Precision[z]<Infinity)

(* BranchCut: type1 *)

flammerWaveR2CosType1[m_,n_,c_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  fwsDiskCos[m,spZ]*FlammerWaveR2[m,n,spC,Cos[spZ]]+
  (Pi/2*JoiningFactorKappa1[m,n,spC]*
  (fwsDiskCos[m-1,spZ]+I*fwsDiskCos[m,spZ])*FlammerWaveR1[m,n,spC,Cos[spZ]]+
  Pi/2*(-1)^m*fwsDiskCos[-1,spZ]*legenPTermF[m,n,spC,Cos[spZ]])/
  JoiningFactorKappa2[m,n,spC]]*Exp[3 m Pi I/2];

(* BranchCut: type2 *)

flammerWaveR2CosType2[m_,n_,c_,z_]:=
  If[Im[z]>=0,FlammerWaveR2[m,n,c,Cos[z]],
  If[Re[Sin[z]]>=0,flammWavR2Cos2[m,n,c,z],
  If[OddQ[n],flammWavR2Cos2[m,n,c,z+Pi],-flammWavR2Cos2[m,n,c,z+Pi]]]]/;
  (N[Abs[c]]=!=0.)

flammWavR2Cos2[m_,n_,c_,z_]:=
  Exp[3 m Pi I/2]/JoiningFactorKappa2[m,n,c]*
  (FlammerWaveS2[m,n,c,Cos[z]]+I*(Pi/2)*FlammerWaveS1[m,n,c,Cos[z]])

(**************************************************************)
(*           spherHankelH1[n,z], spherHankelH2[n,z]           *)
(**************************************************************)

spherHankelH1[n_,z_]:=
  Sum[(n+r)!/(r!*(n-r)!)*(I/(2 z))^r,{r,0,n}]*Exp[I z]/z*(-I)^(n+1);

spherHankelH2[n_,z_]:=
  Sum[(n+r)!/(r!*(n-r)!)*(-I/(2 z))^r,{r,0,n}]*Exp[-I z]/z*I^(n+1);

(**************************************************************)
(*                     asymFR3R4Cfit[n,z]                     *)
(**************************************************************)

asymFR3R4Cfit[n_,c_]:=
  Module[{t=Abs[c]},9.29525671784882-11.549251663568597*t^(1/3)+
  5.996064718845773*Sqrt[t]-0.2271089371207161*t-
  0.005036099179146592*n^(1/3)-0.02899932477508569*Sqrt[n]+
  0.018521663663984774*n-0.000648174463362847*t*n-
  0.07245897792150502*(t*n)^(1/3)+0.05508060049949158*Sqrt[t*n]];

(**************************************************************)
(*                 FlammerWaveR3Cos[m,n,c,z]                  *)
(**************************************************************)

FlammerWaveR3Cos[m_,n_,c_,z_,opts___?OptionQ]:=
  Module[{optf},
  optf=BranchCut/.{opts}/.Options[FlammerWaveR3Cos];
  If[optf==1,flammerWaveR3CosType1[m,n,c,z],
             flammerWaveR3CosType2[m,n,c,z]]]/;(Precision[z]<Infinity)

asymflaR3Cos[m_,n_,c_,z_]:=
  Sum[I^(r+m-n)*(2 m+r)!/r!*FlammerCoefficientD[m,n,c,r]*
  spherHankelH1[m+r,c*Cos[z]],{r,0,20,2}]*
  ((Cos[z]^2-1)/Cos[z]^2)^(m/2)/radialFactorRho[m,n,c]/;(EvenQ[n-m])

asymflaR3Cos[m_,n_,c_,z_]:=
  Sum[I^(r+m-n)*(2 m+r)!/r!*FlammerCoefficientD[m,n,c,r]*
  spherHankelH1[m+r,c*Cos[z]],{r,1,21,2}]*
  ((Cos[z]^2-1)/Cos[z]^2)^(m/2)/radialFactorRho[m,n,c]/;(OddQ[n-m])

(* BranchCut: type1 *)

asymCosSin[c_,z_]:=If[N[Im[c]]===0.,Cos[z],-Sin[z]];

flammerWaveR3CosType1[m_,n_,c_,z_]:=
  If[(Im[asymCosSin[c,z]]>N[asymFR3R4Cfit[n,c]] && Im[z]>0),
  asymflaR3Cos[m,n,c,z],
  (I*fwsDiskCos[m,z]*FlammerWaveR2[m,n,c,Cos[z]]+
  (r1Cosfactor[m,z]*Exp[-3 m Pi I/2]+Pi/2*jfKappa1parKappa2[m,n,c]*
  (I*fwsDiskCos[m-1,z]-fwsDiskCos[m,z]))*FlammerWaveR1[m,n,c,Cos[z]]+
  I*Pi/2*(-1)^m*fwsDiskCos[-1,z]*legenPTermF[m,n,c,Cos[z]]/
  JoiningFactorKappa2[m,n,c])*Exp[3 m Pi I/2]];

r1Cosfactor[m_,z_]:=If[(OddQ[m] && Im[z]<=0),-1,1];

(* BranchCut: type2 *)

flammerWaveR3CosType2[m_,n_,c_,z_]:=
  If[(Im[Cos[z]]>N[asymFR3R4Cfit[n,c]] && Im[z]>0),asymflaR3Cos[m,n,c,z],
  If[Im[z]>=0,
  FlammerWaveR1[m,n,c,Cos[z]]+I*FlammerWaveR2[m,n,c,Cos[z]],
  If[Re[Sin[z]]>=0,
  FlammerWaveR1Cos[m,n,c,z]+I*flammWavR2Cos2[m,n,c,z],
  If[OddQ[n],
  FlammerWaveR1Cos[m,n,c,z]+I*flammWavR2Cos2[m,n,c,z+Pi],
  FlammerWaveR1Cos[m,n,c,z]-I*flammWavR2Cos2[m,n,c,z+Pi]]]]]/;
  (N[Abs[c]]=!=0.)

(**************************************************************)
(*                 FlammerWaveR4Cos[m,n,c,z]                  *)
(**************************************************************)

FlammerWaveR4Cos[m_,n_,c_,z_,opts___?OptionQ]:=
  Module[{optf},
  optf=BranchCut/.{opts}/.Options[FlammerWaveR4Cos];
  If[optf==1,flammerWaveR4CosType1[m,n,c,z],
             flammerWaveR4CosType2[m,n,c,z]]]/;(Precision[z]<Infinity)

asymflaR4Cos[m_,n_,c_,z_]:=
  Sum[I^(r+m-n)*(2 m+r)!/r!*FlammerCoefficientD[m,n,c,r]*
  spherHankelH2[m+r,c*Cos[z]],{r,0,20,2}]*
  ((Cos[z]^2-1)/Cos[z]^2)^(m/2)/radialFactorRho[m,n,c]/;(EvenQ[n-m])

asymflaR4Cos[m_,n_,c_,z_]:=
  Sum[I^(r+m-n)*(2 m+r)!/r!*FlammerCoefficientD[m,n,c,r]*
  spherHankelH2[m+r,c*Cos[z]],{r,1,21,2}]*
  ((Cos[z]^2-1)/Cos[z]^2)^(m/2)/radialFactorRho[m,n,c]/;(OddQ[n-m])

(* BranchCut: type1 *)

flammerWaveR4CosType1[m_,n_,c_,z_]:=
  If[(Im[asymCosSin[c,z]]<N[-asymFR3R4Cfit[n,c]] && Im[z]>0),
  asymflaR4Cos[m,n,c,z],
  (-I*fwsDiskCos[m,z]*FlammerWaveR2[m,n,c,Cos[z]]+
  (r1Cosfactor[m,z]*Exp[-3 m Pi I/2]-Pi/2*jfKappa1parKappa2[m,n,c]*
  (I*fwsDiskCos[m-1,z]-fwsDiskCos[m,z]))*FlammerWaveR1[m,n,c,Cos[z]]-
  I*Pi/2*(-1)^m*fwsDiskCos[-1,z]*legenPTermF[m,n,c,Cos[z]]/
  JoiningFactorKappa2[m,n,c])*Exp[3 m Pi I/2]];

(* BranchCut: type2 *)

flammerWaveR4CosType2[m_,n_,c_,z_]:=
  If[(Im[Cos[z]]<N[-asymFR3R4Cfit[n,c]] && Im[z]>0),asymflaR4Cos[m,n,c,z],
  If[Im[z]>=0,
  FlammerWaveR1[m,n,c,Cos[z]]-I*FlammerWaveR2[m,n,c,Cos[z]],
  If[Re[Sin[z]]>=0,
  FlammerWaveR1Cos[m,n,c,z]-I*flammWavR2Cos2[m,n,c,z],
  If[OddQ[n],
  FlammerWaveR1Cos[m,n,c,z]-I*flammWavR2Cos2[m,n,c,z+Pi],
  FlammerWaveR1Cos[m,n,c,z]+I*flammWavR2Cos2[m,n,c,z+Pi]]]]]/;
  (N[Abs[c]]=!=0.)



(*---------------------------------------------------------------------------*)
(*                                                                           *)
(*                                                                           *)
(*                     Spheroidal wave Cosine functions                      *)
(*                         (Meixner-Schafke scheme)                          *)
(*                                                                           *)
(*                                                                           *)
(*---------------------------------------------------------------------------*)

(*****************************************************************************)
(*                       Spheroidal WavePSCos[n,m,g,z]                        *)
(*****************************************************************************)

SpheroidalWavePSCos[n_,m_,g_,z_]:=
  Module[{spG,spZ,ans},
    {spG,spZ}=setPrecSph[{g,z}];
    ans=(-1)^m*FlammerWaveS1Cos[m,n,spG,spZ]*Sqrt[Abs[2/(2 n+1)*Gamma[n+m+1]/
      Gamma[n-m+1]/FlammerNormFactorN[m,n,spG]]];
    If[Abs[Im[z]]>0,ans,Chop[ans,10^-10]]]/;
  (Precision[z]<Infinity && N[Abs[g]]=!=0.)

(*****************************************************************************)
(*                       SpheroidalWaveQSCos[n,m,g,z]                        *)
(*****************************************************************************)

SpheroidalWaveQSCos[n_,m_,g_,z_]:=
  Module[{spG,spZ,ans},
    {spG,spZ}=setPrecSph[{g,z}];
    ans=(-1)^m*FlammerWaveS2Cos[m,n,spG,spZ]*Sqrt[Abs[2/(2 n+1)*Gamma[n+m+1]/
      Gamma[n-m+1]/FlammerNormFactorN[m,n,spG]]];
    If[Abs[Im[z]]>0,ans,Chop[ans,10^-10]]]/;
  (Precision[z]<Infinity && N[Abs[g]]=!=0.)

(*****************************************************************************)
(*                       SpheroidalWaveS1Cos[n,m,g,z]                        *)
(*****************************************************************************)

SpheroidalWaveS1Cos[n_,m_,g_,z_]:=
  FlammerWaveR1Cos[m,n,g,z]/;(Precision[z]<Infinity)

(*****************************************************************************)
(*                       SpheroidalWaveS2Cos[n,m,g,z]                        *)
(*****************************************************************************)

SpheroidalWaveS2Cos[n_,m_,g_,z_,opts___?OptionQ]:=
  Module[{optf},
  optf=BranchCut/.{opts}/.Options[SpheroidalWaveS2Cos];
  If[optf==1,flammerWaveR2CosType1[m,n,g,z],
             flammerWaveR2CosType2[m,n,g,z]]]/;(Precision[z]<Infinity)

(*****************************************************************************)
(*                       SpheroidalWaveS3Cos[n,m,g,z]                        *)
(*****************************************************************************)

SpheroidalWaveS3Cos[n_,m_,g_,z_,opts___?OptionQ]:=
  Module[{optf},
  optf=BranchCut/.{opts}/.Options[SpheroidalWaveS3Cos];
  If[optf==1,flammerWaveR3CosType1[m,n,g,z],
             flammerWaveR3CosType2[m,n,g,z]]]/;(Precision[z]<Infinity)

(*****************************************************************************)
(*                       SpheroidalWaveS4Cos[n,m,g,z]                        *)
(*****************************************************************************)

SpheroidalWaveS4Cos[n_,m_,g_,z_,opts___?OptionQ]:=
  Module[{optf},
  optf=BranchCut/.{opts}/.Options[SpheroidalWaveS4Cos];
  If[optf==1,flammerWaveR4CosType1[m,n,g,z],
             flammerWaveR4CosType2[m,n,g,z]]]/;(Precision[z]<Infinity)



(*---------------------------------------------------------------------------*)
(*                                                                           *)
(*                                                                           *)
(*                     Spheroidal wave Cosine functions                      *)
(*                (Stratton-Morse-Chu-Little-Corbato scheme)                 *)
(*                                                                           *)
(*                                                                           *)
(*---------------------------------------------------------------------------*)

(**************************************************************)
(*                 StrattonWaveS1Cos[m,n,c,z]                 *)
(**************************************************************)

StrattonWaveS1Cos[m_,n_,c_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  FlammerWaveS1Cos[m,n,spC,spZ] Gamma[n+m+1]/
  Gamma[n-m+1]/radialFactorRho[m,n,spC]]/;
  (Precision[z]<Infinity)

(**************************************************************)
(*                 StrattonWaveS2Cos[m,n,c,z]                 *)
(**************************************************************)

StrattonWaveS2Cos[m_,n_,c_,z_]:=
  Module[{spC,spZ},{spC,spZ}=setPrecSph[{c,z}];
  FlammerWaveS2Cos[m,n,spC,spZ] Gamma[n+m+1]/
  Gamma[n-m+1]/radialFactorRho[m,n,spC]]/;
  (Precision[z]<Infinity)



(*---------------------------------------------------------------------------*)
(*                                                                           *)
(*                                                                           *)
(*                   Liouville's spheroidal wave functions                   *)
(*                            (Flammer scheme)                               *)
(*                                                                           *)
(*                                                                           *)
(*---------------------------------------------------------------------------*)

(**************************************************************)
(*                         sqrtSin[z]                         *)
(**************************************************************)

sqrtSin[z_]:=Sqrt[Sin[z]]/;(N[-Pi/2]<Re[z]<=N[3 Pi/2])

sqrtSin[z_]:=-sqrtSin[z-2 Pi]/;(Re[z]>N[3 Pi/2])

sqrtSin[z_]:=-sqrtSin[z+2 Pi]/;(Re[z]<=N[-Pi/2])

(**************************************************************)
(*                        sqrtSinh[z]                         *)
(**************************************************************)

sqrtSinh[z_]:=Sqrt[Sinh[z]]/;(N[-Pi]<Im[z]<=N[Pi])

sqrtSinh[z_]:=-sqrtSinh[z-2 I Pi]/;(Im[z]>N[Pi])

sqrtSinh[z_]:=-sqrtSinh[z+2 I Pi]/;(Im[z]<=N[-Pi])

(**************************************************************)
(*                        sqrtSinh2[z]                        *)
(**************************************************************)

sqSinhMinu[z_]:=Sqrt[Sinh[z]]/;(N[0]<Im[z]<=N[2 Pi])

sqSinhMinu[z_]:=-sqSinhMinu[z-2 I Pi]/;(Im[z]>N[2 Pi])

sqSinhMinu[z_]:=-sqSinhMinu[z+2 I Pi]/;(Im[z]<=N[0])

sqrtSinh2[z_]:=sqrtSinh[z]/;(Re[z]>=0)

sqrtSinh2[z_]:=-sqSinhMinu[z]/;(Re[z]<0)

(**************************************************************)
(*                    LiouvilleP[m,n,c,z]                     *)
(**************************************************************)

LiouvilleP[m_,n_,c_,z_]:=
  FlammerWaveS1Cos[m,n,c,z]/sqrtSin[z]/;
  (Precision[z]<Infinity)

(**************************************************************)
(*                    LiouvilleQ[m,n,c,z]                     *)
(**************************************************************)

LiouvilleQ[m_,n_,c_,z_]:=
  FlammerWaveS2Cos[m,n,c,z]/sqrtSin[z]/;
  (Precision[z]<Infinity)

(**************************************************************)
(*                    LiouvilleJ[m,n,c,z]                     *)
(**************************************************************)

LiouvilleJ[m_,n_,c_,z_,opts___?OptionQ]:=
  Module[{optf},
  optf=BranchCut/.{opts}/.Options[LiouvilleJ];
  If[optf==1,liouvilleJType1[m,n,c,z],
             liouvilleJType2[m,n,c,z]]]/;(Precision[z]<Infinity)

(* BranchCut: type1 *)

liouvilleJType1[m_,n_,c_,z_]:=
  FlammerWaveR1Cos[m,n,c,I z]/sqrtSinh2[z];

(* BranchCut: type2 *)

liouvilleJType2[m_,n_,c_,z_]:=
  FlammerWaveR1Cos[m,n,c,I z]/sqrtSinh[z];

(**************************************************************)
(*                    LiouvilleY[m,n,c,z]                     *)
(**************************************************************)

LiouvilleY[m_,n_,c_,z_,opts___?OptionQ]:=
  Module[{optf},
  optf=BranchCut/.{opts}/.Options[LiouvilleY];
  If[optf==1,liouvilleYType1[m,n,c,z],
             liouvilleYType2[m,n,c,z]]]/;(Precision[z]<Infinity)

(* BranchCut: type1 *)

liouvilleYType1[m_,n_,c_,z_]:=
  FlammerWaveR2Cos[m,n,c,I z,BranchCut->1]/sqrtSinh2[z];

(* BranchCut: type2 *)

liouvilleYType2[m_,n_,c_,z_]:=
  FlammerWaveR2Cos[m,n,c,I z,BranchCut->2]/sqrtSinh[z];

(**************************************************************)
(*                    LiouvilleH1[m,n,c,z]                    *)
(**************************************************************)

(*
LiouvilleH1[m_,n_,c_,z_,opts___?OptionQ]:=
  Module[{optf},
  optf=BranchCut/.{opts}/.Options[LiouvilleH1];
  If[optf==1,LiouvilleJ[m,n,c,z,BranchCut->1]+I LiouvilleY[m,n,c,z,BranchCut->1],
             LiouvilleJ[m,n,c,z,BranchCut->2]+I LiouvilleY[m,n,c,z,BranchCut->2]]]/;
  (Precision[z]<Infinity)
*)

LiouvilleH1[m_,n_,c_,z_,opts___?OptionQ]:=
  Module[{optf},
  optf=BranchCut/.{opts}/.Options[LiouvilleH1];
  If[optf==1,liouvilleH1Type1[m,n,c,z],
             liouvilleH1Type2[m,n,c,z]]]/;(Precision[z]<Infinity)

(* BranchCut: type1 *)

liouvilleH1Type1[m_,n_,c_,z_]:=
  FlammerWaveR3Cos[m,n,c,I z,BranchCut->1]/sqrtSinh2[z];

(* BranchCut: type2 *)

liouvilleH1Type2[m_,n_,c_,z_]:=
  FlammerWaveR3Cos[m,n,c,I z,BranchCut->2]/sqrtSinh[z];

(**************************************************************)
(*                    LiouvilleH2[m,n,c,z]                    *)
(**************************************************************)

(*
LiouvilleH2[m_,n_,c_,z_,opts___?OptionQ]:=
  Module[{optf},
  optf=BranchCut/.{opts}/.Options[LiouvilleH2];
  If[optf==1,LiouvilleJ[m,n,c,z,BranchCut->1]-I LiouvilleY[m,n,c,z,BranchCut->1],
             LiouvilleJ[m,n,c,z,BranchCut->2]-I LiouvilleY[m,n,c,z,BranchCut->2]]]/;
  (Precision[z]<Infinity)
*)

LiouvilleH2[m_,n_,c_,z_,opts___?OptionQ]:=
  Module[{optf},
  optf=BranchCut/.{opts}/.Options[LiouvilleH2];
  If[optf==1,liouvilleH2Type1[m,n,c,z],
             liouvilleH2Type2[m,n,c,z]]]/;(Precision[z]<Infinity)

(* BranchCut: type1 *)

liouvilleH2Type1[m_,n_,c_,z_]:=
  FlammerWaveR4Cos[m,n,c,I z,BranchCut->1]/sqrtSinh2[z];

(* BranchCut: type2 *)

liouvilleH2Type2[m_,n_,c_,z_]:=
  FlammerWaveR4Cos[m,n,c,I z,BranchCut->2]/sqrtSinh[z];




(*****************************************************************************)

End[]

Protect[SpheroidalEigenvalueLambda, FlammerEigenvalueLambda, 
  JoiningFactorKappa1, JoiningFactorKappa2, 
  FlammerWaveS1, FlammerWaveS2, FlammerWaveR1, FlammerWaveR2, 
  FlammerWaveR3, FlammerWaveR4, 
  SpheroidalWavePS, SpheroidalWaveQS, SpheroidalWaveS1, SpheroidalWaveS2, 
  SpheroidalWaveS3, SpheroidalWaveS4, 
  StrattonWaveS1, StrattonWaveS2, 
  FlammerWaveS1Cos, FlammerWaveS2Cos, FlammerWaveR1Cos, FlammerWaveR2Cos, 
  FlammerWaveR3Cos, FlammerWaveR4Cos, 
  SpheroidalWavePSCos, SpheroidalWaveQSCos, SpheroidalWaveS1Cos, SpheroidalWaveS2Cos, 
  SpheroidalWaveS3Cos, SpheroidalWaveS4Cos, 
  StrattonWaveS1Cos, StrattonWaveS2Cos, 
  LiouvilleP, LiouvilleQ, LiouvilleJ, LiouvilleY, LiouvilleH1, LiouvilleH2]

EndPackage[]
